#! /usr/bin/env python3
import configparser
import logging
import os
import platform
from importlib.metadata import version

from libsrg.Runner import Runner


class Info:
    def __init__(self, hostname: str = None):
        self.logger = logging.getLogger(self.__class__.__name__)

        self.libsrg_ver = version('libsrg')
        self.local_node = platform.node()
        self.local_node_short = self.local_node.split('.')[0]
        self.config = configparser.ConfigParser()
        self.like_fedora=False
        self.like_rhel=False
        self.like_redhat=False
        self.like_debian=False
        self.userat=None
        if hostname:
            self.userat = f"root@{hostname}"
            self.hostname = hostname
        else:
            self.userat = None
            self.hostname = self.local_node_short
        rm = Runner("uname -m",userat=self.userat,rethrow=True,timeout=10,throw=True)
        if not rm.success:
            raise Exception(f"Command Failed {rm}")
        self.machine= rm.so_lines[-1]

        rk = Runner("uname -r",userat=self.userat,rethrow=True,timeout=10,throw=True)
        self.kernel= rk.so_lines[-1]

        rfqdn = Runner("hostname -f", userat=self.userat,rethrow=True,timeout=10,throw=True)
        self.fqdn = rfqdn.so_lines[-1]

        rshort = Runner("hostname -s", userat=self.userat,rethrow=True,timeout=10,throw=True)
        self.short = rshort.so_lines[-1]

        rip = Runner("hostname -i", userat=self.userat,rethrow=True,timeout=10,throw=True)
        self.ip = rip.so_lines[-1]

        r = Runner("cat /etc/os-release", userat=self.userat,rethrow=True,timeout=10,throw=True)
        if not r.success:
            raise Exception(f"os-release failed for {self.userat}")
        data = r.so_lines
        # add a section header
        data.insert(0, "[osrelease]")
        self.config.read_string("\n".join(data))
        # for key in self.config:
        #     self.logger.info(key)
        osrelease = self.config['osrelease']
        self.osrelease = osrelease
        # for key, val in osrelease.items():
        #     self.logger.info(f"{key} = {val}")

        # fedora has 'id' lower case, raspian upper
        # configparser says keys are case insensitive
        self.id = (osrelease['ID']).strip('"\'')
        if 'id' in osrelease:
            self.id = osrelease['id'].strip('"\'')
        else:
            self.id = 'unknown'
            self.logger.error(f"'id' not found in {osrelease}")

        # raspian 'ID_LIKE' says, "But you can call me Debian"
        if 'ID_LIKE' in osrelease:
            self.id_like = (osrelease['ID_LIKE']).strip('"\'')
        else:
            self.id_like = self.id
        # self.logger.info(f"id={self.id}, id_like={self.id_like} ")

        if 'PRETTY_NAME' in osrelease:
            self.pretty_name = (osrelease['PRETTY_NAME']).replace('"', '')
        else:
            self.pretty_name = self.id + " " + osrelease['VERSION_ID']

        if "rhel" in self.id or "rhel" in self.id_like:
            self.like_rhel = True
        elif "fedora" in self.id or "fedora" in self.id_like:
            self.like_fedora = True
        elif "debian" in self.id or "debian" in self.id_like:
            self.like_debian = True
        elif "ubuntu" in self.id or "ubuntu" in self.id_like:
            self.like_debian = True

        self.like_redhat = self.like_fedora or self.like_rhel

        self.uid = os.getuid()
        bios = "ARM" if "aarch64" in self.machine else "BIOS"
        # These next few Runner calls may return non zero status
        rhow = Runner(f"test -d /sys/firmware/efi", userat=self.userat,rethrow=True,timeout=10,throw=False)
        self.uefi = "UEFI" if rhow.success else bios

        r2 = Runner("grep 'exclude=kernel' /etc/dnf/dnf.conf", userat=self.userat,rethrow=True,timeout=10,throw=False)
        ret = r2.ret
        if ret == 0:
            self.kernel_dnf="locked"
        elif ret == 1:
            self.kernel_dnf="unlocked"
        else:
            self.kernel_dnf="NA"

    def __str__(self):
        return f"Info({self.hostname=} {self.id=} {self.id_like=} {self.pretty_name=} {self.machine=})"

    def is_root(self) ->bool:
        return self.uid == 0

    def exit_if_not_root(self):
        if not self.is_root():
            self.logger.critical("Must run as root, uid={self.uid}, hostname={self.hostname}")
            exit(-1)

    def is_x86_64(self) -> bool:
        return self.machine == "x86_64"

    def dump(self):
        #print(self)
        for k, v in info.__dict__.items():
            if k not in ["logger","config","osrelease"]:
                print(f"{k}={v}")
        for k,v in info.osrelease.items():
            print(f"osrelease.{k}={v}")
        print("-----------------------------------------------")


if __name__ == '__main__':
    # info = Info("nas0")
    info = Info()
    info.dump()
    # info = Info(hostname="nas0")
    # info.dump()
    # info = Info(hostname="nas1")
    # info.dump()
    err=Info("xxx")

